local addonName, addon = ...

local plugin = addon:NewModule("Minimap", "AceHook-3.0")
plugin.Order = 1
plugin.Icon = [[Interface\Icons\INV_Offhand_Sunwell_D_01]]

local AceGUI	= LibStub("AceGUI-3.0")
local LSM		= LibStub("LibSharedMedia-3.0")
local FA		= LibStub("LibFrameAnchorRegistry-1.0")

local db
local UIF

local backdrop = {
	insets = {},
	tile = false, 
	tileSize = 0
}

local SettingPoistion = false

function plugin:OnInitialize()
	local defaults = {
		profile = {
			Enabled = true,
			
			Shape = "SQUARE",
			
			Scale = 1.0,
			ClampToScreen = true,
			EnableMouseWheel = true,
			HideZoomButtons = true,
			HideCalendar = true,
			
			Anchor = nil,
			Point = "TOPRIGHT",
			RelPoint = "TOPRIGHT",
			x = 0,
			y = 0,
			
			BackgroundTexture = "Solid",
			BackgroundBorderTexture = "Glowing",
			BackgroundInsets = 0,
			BackgroundPadding = 4,
			BackgroundColor = {0, 0, 0, 1},
			BackgroundBorderColor = {0, 0, 0, 1},
			BackgroundBorderWidth = 4,
		}
	}

	self.db = addon.db:RegisterNamespace(self:GetName(), defaults)
	db = self.db.profile
	
	self.db.RegisterCallback(self, "OnProfileChanged", "ProfileChanged")
	self.db.RegisterCallback(self, "OnProfileCopied", "ProfileChanged")
	self.db.RegisterCallback(self, "OnProfileReset", "ProfileChanged")
	
	self.background = CreateFrame("Frame", "mMinimapBackground", Minimap)
	self.background:SetFrameLevel(MinimapCluster:GetFrameLevel() - 1)
end

function plugin:OnEnable()
	UIF = LibStub("LibGUIFactory-1.0"):GetFactory("mMinimap")
	
	self:SecureHook(MinimapCluster, "SetPoint", "MapSetPoint")
	
	addon.RegisterCallback(self, "ConfigMode")
	FA.RegisterCallback(self, "FrameRegistered", "ApplySettings")
	
	self:ApplySettings()
end

function plugin:OnDisable()
	addon.UnregisterAllCallbacks(self)
end

function plugin:ProfileChanged(event)
	db = self.db.profile
	self:ApplySettings()
end

function plugin:MapSetPoint(...)
	if SettingPoistion then return end
	SettingPoistion = true
	MinimapCluster:SetScale(db.Scale)
	local anchor = FA:GetAnchor(db.Anchor)
	local s = MinimapCluster:GetEffectiveScale()
	MinimapCluster:ClearAllPoints()
	MinimapCluster:SetPoint(db.Point, anchor, db.RelPoint, db.x / s, db.y / s)	
	SettingPoistion = false
end

function plugin:ApplySettings()
	Minimap:SetClampedToScreen(db.ClampToScreen)
	
	-- Mousewheel zoom
	if db.EnableMouseWheel then		
		Minimap:EnableMouseWheel()
		if not self:IsHooked(Minimap, "OnMouseWheel") then
			self:HookScript(Minimap, "OnMouseWheel", function(self, direction)
				if(direction > 0) then
					MinimapZoomIn:Click()
				else
					MinimapZoomOut:Click()
				end
			end)
		end
	else		
		Minimap:EnableMouseWheel(false)
		self:Unhook(Minimap, "OnMouseWheel")
	end
	
	if db.HideZoomButtons then
		MinimapZoomIn:Hide()
		MinimapZoomOut:Hide()
	else
		MinimapZoomIn:Show()
		MinimapZoomOut:Show()
	end
	
	-- Minimap Shape
	if db.Shape == "SQUARE" then
		Minimap:SetMaskTexture([=[Interface\ChatFrame\ChatFrameBackground]=])
	else
		Minimap:SetMaskTexture([=[Textures\MinimapMask]=])
	end
	
	-- World map button
	MiniMapWorldMapButton:Hide()
	
	-- Time/Calendar
	if db.HideCalendar then
		GameTimeFrame:Hide()
	else
		GameTimeFrame:Show()
	end
	
	-- Position	
	MinimapCluster:SetPoint(db.Point)	
	
	-- Background
	local b = self.background
	b:ClearAllPoints()
	b:SetPoint("TOPLEFT", -db.BackgroundPadding, db.BackgroundPadding)
	b:SetPoint("BOTTOMRIGHT", db.BackgroundPadding, -db.BackgroundPadding)
	
	backdrop.insets.left = db.BackgroundInsets
	backdrop.insets.right = db.BackgroundInsets
	backdrop.insets.top = db.BackgroundInsets
	backdrop.insets.bottom = db.BackgroundInsets
	backdrop.edgeFile = LSM:Fetch("border", db.BackgroundBorderTexture)
	backdrop.bgFile = LSM:Fetch("background", db.BackgroundTexture)
	backdrop.edgeSize = db.BackgroundBorderWidth
	b:SetBackdrop(backdrop)
	b:SetBackdropColor(unpack(db.BackgroundColor))
	b:SetBackdropBorderColor(unpack(db.BackgroundBorderColor))
end

function plugin:ConfigMode(event, enable)
end

do
	local function Callback()
		plugin:ApplySettings()
	end
	
	function plugin:GetOptions()
		local g = AceGUI:Create("SimpleGroup")
		g:SetFullWidth(true)
		g:SetLayout("Flow")
		
		g:AddChild(UIF:CheckBox("Mouse zoom", db, "EnableMouseWheel", Callback, nil, 0.33))
		g:AddChild(UIF:CheckBox("Hide zoom buttons", db, "HideZoomButtons", Callback, nil, 0.32))
		g:AddChild(UIF:CheckBox("Hide calendar button", db, "HideCalendar", Callback, nil, 0.33))
		
		local t = AceGUI:Create("FrameAnchorDropdown")
		t:SetRelativeWidth(0.5)
		t:SetList(FA:GetAnchors(MinimapCluster))
		t:SetLabel("Anchor")
		t:SetValue(db.Anchor)
		t:SetCallback("OnValueChanged", function(self, event, cat, name, frame)
			db.x = 0
			db.y = 0
			db.Anchor = cat
			Callback()
		end)
		g:AddChild(t)
		g:AddChild(UIF:Slider("Scale", db, "Scale", 0.1, 5, 0.01, Callback, 0.5))
		g:AddChild(UIF:GridSelect("Point", db, "Point", "Anchor", 3, Callback, 0.5, 75))
		g:AddChild(UIF:GridSelect("Relative Point", db, "RelPoint", "Anchor", 3, Callback, 0.5, 75))
		g:AddChild(UIF:PositionSlider("x", "X", db, "x", 0.1, Callback, 0.44))
		g:AddChild(UIF:RelativeSpacer(0.1))
		g:AddChild(UIF:PositionSlider("y", "Y", db, "y", 0.1, Callback, 0.44))	
		
		g:AddChild(UIF:NewLine(15))
		g:AddChild(UIF:LSMDropdown("border", "Border Texture", db, "BackgroundBorderTexture", Callback, 0.44))
		g:AddChild(UIF:RelativeSpacer(0.1))
		g:AddChild(UIF:LSMDropdown("background", "Background Texture", db, "BackgroundTexture", Callback, 0.44))
		
		g:AddChild(UIF:ColorSelect("Border Color", db, "BackgroundBorderColor", Callback, true, 0.44))
		g:AddChild(UIF:RelativeSpacer(0.1))
		g:AddChild(UIF:ColorSelect("Background Color", db, "BackgroundColor", Callback, true, 0.44))
		
		g:AddChild(UIF:Slider("Border Width", db, "BackgroundBorderWidth", 0, 30, 0.1, Callback, 0.44))
		g:AddChild(UIF:RelativeSpacer(0.1))
		g:AddChild(UIF:Slider("Padding", db, "BackgroundPadding", 0, 30, 0.1, Callback, 0.44))
		g:AddChild(UIF:Slider("Insets", db, "BackgroundInsets", 0, 30, 0.1, Callback, 1))
		
		return g
	end
end

function GetMinimapShape() return db.Shape end